@@ -119,8 +119,7 @@ g_mime_multipart_signed_class_init (GMimeMultipartSignedClass *klass)
 static void
 g_mime_multipart_signed_init (GMimeMultipartSigned *mps, GMimeMultipartSignedClass *klass)
 {
-	mps->protocol = NULL;
-	mps->micalg = NULL;
+	
 }
 
 static void
@@ -128,9 +127,6 @@ g_mime_multipart_signed_finalize (GObject *object)
 {
 	GMimeMultipartSigned *mps = (GMimeMultipartSigned *) object;
 	
-	g_free (mps->protocol);
-	g_free (mps->micalg);
-	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -167,17 +163,6 @@ multipart_signed_remove_header (GMimeObject *object, const char *header)
 static void
 multipart_signed_set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
-	GMimeMultipartSigned *mps = (GMimeMultipartSigned *) object;
-	const char *protocol, *micalg;
-	
-	protocol = g_mime_content_type_get_parameter (content_type, "protocol");
-	g_free (mps->protocol);
-	mps->protocol = g_strdup (protocol);
-	
-	micalg = g_mime_content_type_get_parameter (content_type, "micalg");
-	g_free (mps->micalg);
-	mps->micalg = g_strdup (micalg);
-	
 	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
 }
 
@@ -205,12 +190,13 @@ GMimeMultipartSigned *
 g_mime_multipart_signed_new (void)
 {
 	GMimeMultipartSigned *multipart;
-	GMimeContentType *type;
+	GMimeContentType *content_type;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART_SIGNED, NULL);
+	multipart = g_object_newv (GMIME_TYPE_MULTIPART_SIGNED, 0, NULL);
 	
-	type = g_mime_content_type_new ("multipart", "signed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
+	content_type = g_mime_content_type_new ("multipart", "signed");
+	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_object_unref (content_type);
 	
 	return multipart;
 }
@@ -241,11 +227,10 @@ sign_prepare (GMimeObject *mime_part)
 			return;
 		}
 		
-		n = g_mime_multipart_get_number (multipart);
+		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
 			subpart = g_mime_multipart_get_part (multipart, i);
 			sign_prepare (subpart);
-			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (mime_part)) {
 		subpart = GMIME_MESSAGE_PART (mime_part)->message->mime_part;
@@ -286,9 +271,10 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	GMimeStream *stream, *filtered, *sigstream;
 	GMimeContentType *content_type;
 	GMimeDataWrapper *wrapper;
-	GMimeObject *signature;
+	GMimePart *signature;
 	GMimeFilter *filter;
 	GMimeParser *parser;
+	const char *micalg;
 	int rv;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), -1);
@@ -339,6 +325,13 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_mime_stream_reset (sigstream);
 	g_mime_stream_reset (stream);
 	
+	/* set the multipart/signed protocol and micalg */
+	content_type = g_mime_object_get_content_type (GMIME_OBJECT (mps));
+	g_mime_content_type_set_parameter (content_type, "protocol", ctx->sign_protocol);
+	micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, (GMimeCipherHash) rv));
+	g_mime_content_type_set_parameter (content_type, "micalg", micalg);
+	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
+	
 	/* construct the content part */
 	parser = g_mime_parser_new_with_stream (stream);
 	content = g_mime_parser_construct_part (parser);
@@ -346,40 +339,31 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_object_unref (parser);
 	
 	/* construct the signature part */
-	signature = (GMimeObject *) g_mime_part_new ();
+	content_type = g_mime_content_type_new_from_string (ctx->sign_protocol);
+	signature = g_mime_part_new_with_type (content_type->type, content_type->subtype);
+	g_object_unref (content_type);
+	
 	wrapper = g_mime_data_wrapper_new ();
 	g_mime_data_wrapper_set_stream (wrapper, sigstream);
-	g_mime_part_set_content_object (GMIME_PART (signature), wrapper);
+	g_mime_part_set_content_object (signature, wrapper);
 	g_object_unref (sigstream);
 	g_object_unref (wrapper);
 	
-	mps->protocol = g_strdup (ctx->sign_protocol);
-	mps->micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, (GMimeCipherHash) rv));
-	
-	/* set the content-type of the signature part */
-	content_type = g_mime_content_type_new_from_string (mps->protocol);
-	g_mime_object_set_content_type (signature, content_type);
-	
 	/* FIXME: temporary hack, this info should probably be set in
 	 * the CipherContext class - maybe ::sign can take/output a
 	 * GMimePart instead. */
-	if (!g_ascii_strcasecmp (mps->protocol, "application/pkcs7-signature")) {
-		g_mime_part_set_content_encoding (GMIME_PART (signature), GMIME_CONTENT_ENCODING_BASE64);
-		g_mime_part_set_filename (GMIME_PART (signature), "smime.p7m");
+	if (!g_ascii_strcasecmp (ctx->sign_protocol, "application/pkcs7-signature")) {
+		g_mime_part_set_content_encoding (signature, GMIME_CONTENT_ENCODING_BASE64);
+		g_mime_part_set_filename (signature, "smime.p7m");
 	}
 	
 	/* save the content and signature parts */
 	/* FIXME: make sure there aren't any other parts?? */
-	g_mime_multipart_add_part (GMIME_MULTIPART (mps), content);
-	g_mime_multipart_add_part (GMIME_MULTIPART (mps), signature);
+	g_mime_multipart_add (GMIME_MULTIPART (mps), content);
+	g_mime_multipart_add (GMIME_MULTIPART (mps), (GMimeObject *) signature);
 	g_object_unref (signature);
 	g_object_unref (content);
 	
-	/* set the content-type params for this multipart/signed part */
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "micalg", mps->micalg);
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mps), "protocol", mps->protocol);
-	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
-	
 	return 0;
 }
 
@@ -415,7 +399,7 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (ctx->sign_protocol != NULL, NULL);
 	
-	if (g_mime_multipart_get_number ((GMimeMultipart *) mps) < 2) {
+	if (g_mime_multipart_get_count ((GMimeMultipart *) mps) < 2) {
 		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
 			     "Cannot verify multipart/signed part due to missing subparts.");
 		return NULL;
@@ -438,7 +422,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_object_unref (signature);
 		g_free (content_type);
 		
 		return NULL;
@@ -460,7 +443,6 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_mime_stream_flush (filtered_stream);
 	g_object_unref (filtered_stream);
 	g_mime_stream_reset (stream);
-	g_object_unref (content);
 	
 	/* get the signature stream */
 	wrapper = g_mime_part_get_content_object (GMIME_PART (signature));
@@ -477,17 +459,14 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	}
 	
 	g_mime_stream_reset (sigstream);
-	g_object_unref (signature);
-	g_object_unref (wrapper);
 	
 	/* verify the signature */
-	hash = g_mime_cipher_context_hash_id (ctx, mps->micalg);
+	hash = g_mime_cipher_context_hash_id (ctx, micalg);
 	valid = g_mime_cipher_context_verify (ctx, hash, stream, sigstream, err);
 	
 	d(printf ("attempted to verify:\n----- BEGIN SIGNED PART -----\n%.*s----- END SIGNED PART -----\n",
 		  (int) GMIME_STREAM_MEM (stream)->buffer->len, GMIME_STREAM_MEM (stream)->buffer->data));
 	
-	g_object_unref (sigstream);
 	g_object_unref (stream);
 	
 	return valid;
