@@ -63,12 +63,15 @@ static char *multipart_get_headers (GMimeObject *object);
 static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
 /* GMimeMultipart class methods */
-static void multipart_add_part (GMimeMultipart *multipart, GMimeObject *part);
-static void multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index);
-static gboolean multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
-static GMimeObject *multipart_remove_part_at (GMimeMultipart *multipart, int index);
+static void multipart_clear (GMimeMultipart *multipart);
+static void multipart_add (GMimeMultipart *multipart, GMimeObject *part);
+static void multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part);
+static gboolean multipart_remove (GMimeMultipart *multipart, GMimeObject *part);
+static GMimeObject *multipart_remove_at (GMimeMultipart *multipart, int index);
 static GMimeObject *multipart_get_part (GMimeMultipart *multipart, int index);
-static int multipart_get_number (GMimeMultipart *multipart);
+static gboolean multipart_contains (GMimeMultipart *multipart, GMimeObject *part);
+static int multipart_index_of (GMimeMultipart *multipart, GMimeObject *part);
+static int multipart_get_count (GMimeMultipart *multipart);
 static void multipart_set_boundary (GMimeMultipart *multipart, const char *boundary);
 static const char *multipart_get_boundary (GMimeMultipart *multipart);
 
@@ -120,12 +123,15 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 	object_class->get_headers = multipart_get_headers;
 	object_class->write_to_stream = multipart_write_to_stream;
 	
-	klass->add_part = multipart_add_part;
-	klass->add_part_at = multipart_add_part_at;
-	klass->remove_part = multipart_remove_part;
-	klass->remove_part_at = multipart_remove_part_at;
+	klass->add = multipart_add;
+	klass->clear = multipart_clear;
+	klass->insert = multipart_insert;
+	klass->remove = multipart_remove;
+	klass->remove_at = multipart_remove_at;
 	klass->get_part = multipart_get_part;
-	klass->get_number = multipart_get_number;
+	klass->contains = multipart_contains;
+	klass->index_of = multipart_index_of;
+	klass->get_count = multipart_get_count;
 	klass->set_boundary = multipart_set_boundary;
 	klass->get_boundary = multipart_get_boundary;
 }
@@ -134,7 +140,6 @@ static void
 g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClass *klass)
 {
 	multipart->children = g_ptr_array_new ();
-	multipart->boundary = NULL;
 	multipart->preface = NULL;
 	multipart->postface = NULL;
 }
@@ -145,7 +150,6 @@ g_mime_multipart_finalize (GObject *object)
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
 	guint i;
 	
-	g_free (multipart->boundary);
 	g_free (multipart->preface);
 	g_free (multipart->postface);
 	
@@ -213,13 +217,6 @@ multipart_remove_header (GMimeObject *object, const char *header)
 static void
 multipart_set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
-	GMimeMultipart *multipart = (GMimeMultipart *) object;
-	const char *boundary;
-	
-	boundary = g_mime_content_type_get_parameter (content_type, "boundary");
-	g_free (multipart->boundary);
-	multipart->boundary = g_strdup (boundary);
-	
 	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
 }
 
@@ -234,6 +231,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 {
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
 	ssize_t nwritten, total = 0;
+	const char *boundary;
 	GMimeObject *part;
 	guint i;
 	
@@ -241,8 +239,11 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	 * header (in which case it should already be set... or if
 	 * not, then it's a broken multipart and so we don't want to
 	 * alter it or we'll completely break the output) */
-	if (!multipart->boundary && !g_mime_header_list_has_raw (object->headers))
+	boundary = g_mime_object_get_content_type_parameter (object, "boundary");
+	if (!boundary && !g_mime_header_list_has_raw (object->headers)) {
 		g_mime_multipart_set_boundary (multipart, NULL);
+		boundary = g_mime_object_get_content_type_parameter (object, "boundary");
+	}
 	
 	/* write the content headers */
 	if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
@@ -268,7 +269,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		part = multipart->children->pdata[i];
 		
 		/* write the boundary */
-		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", multipart->boundary)) == -1)
+		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", boundary)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -281,8 +282,8 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	}
 	
 	/* write the end-boundary (but only if a boundary is set) */
-	if (multipart->boundary) {
-		if ((nwritten = g_mime_stream_printf (stream, "\n--%s--\n", multipart->boundary)) == -1)
+	if (boundary) {
+		if ((nwritten = g_mime_stream_printf (stream, "\n--%s--\n", boundary)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -312,13 +313,14 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 GMimeMultipart *
 g_mime_multipart_new (void)
 {
+	GMimeContentType *content_type;
 	GMimeMultipart *multipart;
-	GMimeContentType *type;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
+	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
 	
-	type = g_mime_content_type_new ("multipart", "mixed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
+	content_type = g_mime_content_type_new ("multipart", "mixed");
+	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_object_unref (content_type);
 	
 	return multipart;
 }
@@ -337,13 +339,14 @@ g_mime_multipart_new (void)
 GMimeMultipart *
 g_mime_multipart_new_with_subtype (const char *subtype)
 {
+	GMimeContentType *content_type;
 	GMimeMultipart *multipart;
-	GMimeContentType *type;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
+	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
 	
-	type = g_mime_content_type_new ("multipart", subtype ? subtype : "mixed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
+	content_type = g_mime_content_type_new ("multipart", subtype ? subtype : "mixed");
+	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_object_unref (content_type);
 	
 	return multipart;
 }
@@ -351,7 +354,7 @@ g_mime_multipart_new_with_subtype (const char *subtype)
 
 /**
  * g_mime_multipart_set_preface:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  * @preface: preface
  *
  * Sets the preface on the multipart.
@@ -368,7 +371,7 @@ g_mime_multipart_set_preface (GMimeMultipart *multipart, const char *preface)
 
 /**
  * g_mime_multipart_get_preface:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  *
  * Gets the preface on the multipart.
  *
@@ -385,7 +388,7 @@ g_mime_multipart_get_preface (GMimeMultipart *multipart)
 
 /**
  * g_mime_multipart_set_postface:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  * @postface: postface
  *
  * Sets the postface on the multipart.
@@ -402,7 +405,7 @@ g_mime_multipart_set_postface (GMimeMultipart *multipart, const char *postface)
 
 /**
  * g_mime_multipart_get_postface:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  *
  * Gets the postface on the multipart.
  *
@@ -418,28 +421,54 @@ g_mime_multipart_get_postface (GMimeMultipart *multipart)
 
 
 static void
-multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
+multipart_clear (GMimeMultipart *multipart)
 {
-	g_object_ref (part);
+	int i;
 	
-	g_ptr_array_add (multipart->children, part);
+	for (i = 0; i < (int) multipart->children->len; i++)
+		g_object_unref (multipart->children->pdata[i]);
+	
+	g_ptr_array_set_size (multipart->children, 0);
 }
 
 
 /**
- * g_mime_multipart_add_part:
- * @multipart: multipart
- * @part: mime part
+ * g_mime_multipart_clear:
+ * @multipart: a #GMimeMultipart object
+ *
+ * Removes all subparts from @multipart.
+ **/
+void
+g_mime_multipart_clear (GMimeMultipart *multipart)
+{
+	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
+	
+	GMIME_MULTIPART_GET_CLASS (multipart)->clear (multipart);
+}
+
+
+static void
+multipart_add (GMimeMultipart *multipart, GMimeObject *part)
+{
+	g_ptr_array_add (multipart->children, part);
+	g_object_ref (part);
+}
+
+
+/**
+ * g_mime_multipart_add:
+ * @multipart: a #GMimeMultipart object
+ * @part: a #GMimeObject
  *
  * Adds a mime part to the multipart.
  **/
 void
-g_mime_multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
+g_mime_multipart_add (GMimeMultipart *multipart, GMimeObject *part)
 {
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (GMIME_IS_OBJECT (part));
 	
-	GMIME_MULTIPART_GET_CLASS (multipart)->add_part (multipart, part);
+	GMIME_MULTIPART_GET_CLASS (multipart)->add (multipart, part);
 }
 
 
@@ -464,35 +493,35 @@ ptr_array_insert (GPtrArray *array, guint index, gpointer object)
 }
 
 static void
-multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index)
+multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part)
 {
-	g_object_ref (part);
-	
 	ptr_array_insert (multipart->children, index, part);
+	g_object_ref (part);
 }
 
 
 /**
- * g_mime_multipart_add_part_at:
- * @multipart: multipart
+ * g_mime_multipart_insert:
+ * @multipart: a #GMimeMultipart object
  * @part: mime part
  * @index: position to insert the mime part
  *
- * Adds a mime part to the multipart at the position @index.
+ * Inserts the specified mime part into the multipart at the position
+ * @index.
  **/
 void
-g_mime_multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index)
+g_mime_multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part)
 {
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (GMIME_IS_OBJECT (part));
 	g_return_if_fail (index >= 0);
 	
-	GMIME_MULTIPART_GET_CLASS (multipart)->add_part_at (multipart, part, index);
+	GMIME_MULTIPART_GET_CLASS (multipart)->insert (multipart, index, part);
 }
 
 
 static gboolean
-multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
+multipart_remove (GMimeMultipart *multipart, GMimeObject *part)
 {
 	if (!g_ptr_array_remove (multipart->children, part))
 		return FALSE;
@@ -504,8 +533,8 @@ multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 
 
 /**
- * g_mime_multipart_remove_part:
- * @multipart: multipart
+ * g_mime_multipart_remove:
+ * @multipart: a #GMimeMultipart object
  * @part: mime part
  *
  * Removes the specified mime part from the multipart.
@@ -513,17 +542,17 @@ multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
  * Returns: %TRUE if the part was removed or %FALSE otherwise.
  **/
 gboolean
-g_mime_multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
+g_mime_multipart_remove (GMimeMultipart *multipart, GMimeObject *part)
 {
-	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
-	g_return_if_fail (GMIME_IS_OBJECT (part));
+	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), FALSE);
+	g_return_val_if_fail (GMIME_IS_OBJECT (part), FALSE);
 	
-	return GMIME_MULTIPART_GET_CLASS (multipart)->remove_part (multipart, part);
+	return GMIME_MULTIPART_GET_CLASS (multipart)->remove (multipart, part);
 }
 
 
 static GMimeObject *
-multipart_remove_part_at (GMimeMultipart *multipart, int index)
+multipart_remove_at (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
 	
@@ -539,21 +568,22 @@ multipart_remove_part_at (GMimeMultipart *multipart, int index)
 
 
 /**
- * g_mime_multipart_remove_part_at:
- * @multipart: multipart
+ * g_mime_multipart_remove_at:
+ * @multipart: a #GMimeMultipart object
  * @index: position of the mime part to remove
  *
  * Removes the mime part at position @index from the multipart.
  *
- * Returns: the mime part that was removed.
+ * Returns: the mime part that was removed or %NULL if the part was
+ * not contained within the multipart.
  **/
 GMimeObject *
-g_mime_multipart_remove_part_at (GMimeMultipart *multipart, int index)
+g_mime_multipart_remove_at (GMimeMultipart *multipart, int index)
 {
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), NULL);
 	g_return_val_if_fail (index >= 0, NULL);
 	
-	return GMIME_MULTIPART_GET_CLASS (multipart)->remove_part_at (multipart, index);
+	return GMIME_MULTIPART_GET_CLASS (multipart)->remove_at (multipart, index);
 }
 
 
@@ -566,7 +596,6 @@ multipart_get_part (GMimeMultipart *multipart, int index)
 		return NULL;
 	
 	part = multipart->children->pdata[index];
-	g_object_ref (part);
 	
 	return part;
 }
@@ -574,7 +603,7 @@ multipart_get_part (GMimeMultipart *multipart, int index)
 
 /**
  * g_mime_multipart_get_part:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  * @index: position of the mime part
  *
  * Gets the mime part at position @index within the multipart.
@@ -591,27 +620,94 @@ g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
 }
 
 
+static gboolean
+multipart_contains (GMimeMultipart *multipart, GMimeObject *part)
+{
+	int i;
+	
+	for (i = 0; i < (int) multipart->children->len; i++) {
+		if (part == (GMimeObject *) multipart->children->pdata[i])
+			return TRUE;
+	}
+	
+	return FALSE;
+}
+
+
+/**
+ * g_mime_multipart_contains:
+ * @multipart: a #GMimeMultipart object
+ * @part: mime part
+ *
+ * Checks if @part is contained within @multipart.
+ *
+ * Returns: %TRUE if @part is a subpart of @multipart or %FALSE
+ * otherwise.
+ **/
+gboolean
+g_mime_multipart_contains (GMimeMultipart *multipart, GMimeObject *part)
+{
+	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), FALSE);
+	g_return_val_if_fail (GMIME_IS_OBJECT (part), FALSE);
+	
+	return GMIME_MULTIPART_GET_CLASS (multipart)->contains (multipart, part);
+}
+
+
 static int
-multipart_get_number (GMimeMultipart *multipart)
+multipart_index_of (GMimeMultipart *multipart, GMimeObject *part)
+{
+	int i;
+	
+	for (i = 0; i < (int) multipart->children->len; i++) {
+		if (part == (GMimeObject *) multipart->children->pdata[i])
+			return i;
+	}
+	
+	return -1;
+}
+
+
+/**
+ * g_mime_multipart_index_of:
+ * @multipart: a #GMimeMultipart object
+ * @part: mime part
+ *
+ * Gets the index of @part within @multipart.
+ *
+ * Returns: the index of @part within @multipart or %-1 if not found.
+ **/
+int
+g_mime_multipart_index_of (GMimeMultipart *multipart, GMimeObject *part)
+{
+	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), -1);
+	g_return_val_if_fail (GMIME_IS_OBJECT (part), -1);
+	
+	return GMIME_MULTIPART_GET_CLASS (multipart)->index_of (multipart, part);
+}
+
+
+static int
+multipart_get_count (GMimeMultipart *multipart)
 {
 	return multipart->children->len;
 }
 
 
 /**
- * g_mime_multipart_get_number:
- * @multipart: multipart
+ * g_mime_multipart_get_count:
+ * @multipart: a #GMimeMultipart object
  *
  * Gets the number of mime parts contained within the multipart.
  *
  * Returns: the number of mime parts contained within the multipart.
  **/
 int
-g_mime_multipart_get_number (GMimeMultipart *multipart)
+g_mime_multipart_get_count (GMimeMultipart *multipart)
 {
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), -1);
 	
-	return GMIME_MULTIPART_GET_CLASS (multipart)->get_number (multipart);
+	return GMIME_MULTIPART_GET_CLASS (multipart)->get_count (multipart);
 }
 
 
@@ -662,16 +758,13 @@ multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 		boundary = bbuf;
 	}
 	
-	g_free (multipart->boundary);
-	multipart->boundary = g_strdup (boundary);
-	
 	g_mime_object_set_content_type_parameter (GMIME_OBJECT (multipart), "boundary", boundary);
 }
 
 
 /**
  * g_mime_multipart_set_boundary:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  * @boundary: boundary or %NULL to autogenerate one
  *
  * Sets @boundary as the boundary on the multipart. If @boundary is
@@ -689,16 +782,21 @@ g_mime_multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 static const char *
 multipart_get_boundary (GMimeMultipart *multipart)
 {
-	if (!multipart->boundary)
-		multipart_set_boundary (multipart, NULL);
+	GMimeObject *object = (GMimeObject *) multipart;
+	const char *boundary;
 	
-	return multipart->boundary;
+	if ((boundary = g_mime_object_get_content_type_parameter (object, "boundary")))
+		return boundary;
+	
+	multipart_set_boundary (multipart, NULL);
+	
+	return g_mime_object_get_content_type_parameter (object, "boundary");
 }
 
 
 /**
  * g_mime_multipart_get_boundary:
- * @multipart: multipart
+ * @multipart: a #GMimeMultipart object
  *
  * Gets the boundary on the multipart. If the internal boundary is
  * %NULL, then an auto-generated boundary will be set on the multipart
@@ -715,27 +813,37 @@ g_mime_multipart_get_boundary (GMimeMultipart *multipart)
 }
 
 
-/**
- * g_mime_multipart_foreach: 
- * @multipart: a multipart
- * @callback: function to call for @multipart and all of its subparts
- * @user_data: extra data to pass to the callback
- * 
- * Calls @callback on each of @multipart's subparts.
- **/
-void
-g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpointer user_data)
+static void
+multipart_foreach (GMimeMultipart *multipart, GMimeObjectForeachFunc callback, gpointer user_data)
 {
 	GMimeObject *part;
 	guint i;
 	
+	for (i = 0; i < multipart->children->len; i++) {
+		part = (GMimeObject *) multipart->children->pdata[i];
+		callback ((GMimeObject *) multipart, part, user_data);
+		
+		if (GMIME_IS_MULTIPART (part))
+			multipart_foreach ((GMimeMultipart *) part, callback, user_data);
+	}
+}
+
+
+/**
+ * g_mime_multipart_foreach: 
+ * @multipart: a #GMimeMultipart
+ * @callback: function to call for each of @multipart's subparts.
+ * @user_data: user-supplied callback data
+ * 
+ * Recursively calls @callback on each of @multipart's subparts.
+ **/
+void
+g_mime_multipart_foreach (GMimeMultipart *multipart, GMimeObjectForeachFunc callback, gpointer user_data)
+{
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (callback != NULL);
 	
-	for (i = 0; i < multipart->children->len; i++) {
-		part = multipart->children->pdata[i];
-		callback (part, user_data);
-	}
+	multipart_foreach (multipart, callback, user_data);
 }
 
 
@@ -747,7 +855,7 @@ g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpo
  * Gets the mime part with the content-id @content_id from the
  * multipart @multipart.
  *
- * Returns: the GMimeObject whose content-id matches the search string,
+ * Returns: the #GMimeObject whose content-id matches the search string,
  * or %NULL if a match cannot be found.
  **/
 GMimeObject *
@@ -761,21 +869,19 @@ g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const c
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), NULL);
 	g_return_val_if_fail (content_id != NULL, NULL);
 	
-	if (object->content_id && !strcmp (object->content_id, content_id)) {
-		g_object_ref (object);
+	if (object->content_id && !strcmp (object->content_id, content_id))
 		return object;
-	}
 	
 	for (i = 0; i < multipart->children->len; i++) {
 		subpart = multipart->children->pdata[i];
 		
+		if (subpart->content_id && !strcmp (subpart->content_id, content_id))
+			return subpart;
+		
 		if (GMIME_IS_MULTIPART (subpart)) {
 			mpart = (GMimeMultipart *) subpart;
 			if ((part = g_mime_multipart_get_subpart_from_content_id (mpart, content_id)))
 				return part;
-		} else if (subpart->content_id && !strcmp (subpart->content_id, content_id)) {
-			g_object_ref (subpart);
-			return subpart;
 		}
 	}
 	
